
/***********************************************************************/
/*                             fe2d_move.c                             */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*    Demoprogramm, bei dem eine Szene aufgebaut wird, und in der man  */
/*  mit einem Button eine Sinusfoermige Bewegung starten kann.         */
/*    Zur besseren Veranschaulichung rotiert man die Ebene um x oder   */
/*  liest den Status 'fe2d_move.st' ein und waehlt "height (z-value)"  */
/*  als 1.0.                                                           */
/*    Neben dem Einhaengen eines Buttons und Rulers wird zudem demon-  */
/*  striert, wie man die Einstellungen eines Colorbars vom Programm    */
/*  aus vornehmen kann.                                                */
/*                                         RALPH SCHWOERER (27.07.95)  */
/***********************************************************************/

#include <grape.h>

static float movetime = 0.0;
static int steps = 1;
static float fx(x,y) float x,y; { return(x);}
static float fy(x,y) float x,y; { return(y);}



float calc_sinus(float x, float y)
{
  return(g_sin(5.0*(g_sqrt(x*x + y*y) + movetime))*0.5);
}



/*---  Funktion, die die Werte durch lineare Interpolation aus den  ---*/
/*---  Werten in den Ecken jedes Dreiecks ermittelt:                ---*/

void function(FE2D *fe, int i, float bary[3], float *v)
{ 
  float *data;

  data = (float *)fe->data;
  
  *v = data[fe->vertex[i][0]] * bary[0] +
       data[fe->vertex[i][1]] * bary[1] +
       data[fe->vertex[i][2]] * bary[2];
}



SCENE *scene_start_move()
{
  SCENE         *sc;
  FE2D          *fe;
  GRAPHICDEVICE *dev;
  float         *data;
  int           i, count, flag;


  sc = (SCENE *)START_METHOD(G_INSTANCE);
  ASSURE(sc,"Keine Szene gefunden",END_METHOD(NIL));
  fe = (FE2D *)sc->object;
  ASSURE(sc,"Kein Objekt (Fe2d) gefunden",END_METHOD(sc));
  dev = (GRAPHICDEVICE *)GRAPE(GraphicDevice,"get-stddev")();
  dev->attribute(G_MODE_GET,G_DOUBLEBUFFER,&flag);

  data = (float *)mem_alloc(sizeof(float) * fe->number_of_points);

  fe->data = (char *)data;


  /*---  Mache 'steps' Bewegungsschritte:  ---*/
  for (count=0; count<steps; count++) {

    /*---  Berechnen der Werte in jedem Punkt:  ---*/
    for (i=0; i<fe->number_of_points; i++)
      data[i] = calc_sinus(fe->x[i],fe->y[i]);

    /*---  Update des Output-Fensters:  ---*/
    dev->clear();
    GRAPE(fe,sc->method_name)();
    if (flag) dev->attribute(G_MODE_SWAP,G_DOUBLEBUFFER,NIL);

    movetime += 0.01;
  }


  END_METHOD(sc)
}



main()
{  
  SCENE       *sc;
  FE2D        *fe;
  MANAGER     *mgr;
  BUTTON      *button_start;
  RULER       *ruler_steps;
  VEC3        xyz = {0.5, -0.9, 0.0};
  extern FE2D *fe2d_isoline_disp();


  printf("\n\n*----------------------------------------------------*");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Choose  'read st'  'fe2d_move.st'             |");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Set  'height (z-value)'  to  1.0              |");
  printf(  "\n|                                                    |");
  printf(  "\n|   (+  Choose  'number of steps')                   |");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Click the Button 'start the motion'           |");
  printf(  "\n|                                                    |");
  printf(  "\n*----------------------------------------------------*\n\n");

  
  /*---  Bekanntgeben der Methode "start-move" auf der Klasse 'Scene':  ---*/
  GRAPE(Scene,"add-method")("start-move",scene_start_move);


  /*---  Festsetzen der Einstellungen fuer den Colorbar zur Isolinien-  ---*/
  /*---  methode:  Position      = xyz                                  ---*/
  /*---            Hoehe, Breite = 0.4, 0.1                             ---*/
  /*---            Min, Max      = -0.5, 0.5                            ---*/
  GRAPE(Colorbar, "set-default")(xyz, 0.4 , 0.1, 1, -0.5, 0.5, 0, NIL,
		                 fe2d_isoline_disp, "iso2dclb"); 

  /*---  Aufbauen der Fe2d-Instanz:  ---*/
  fe = (FE2D *)GRAPE(Fe2d, "explicit")
       ("plane", fx, fy, NIL, 0., 1., 0., 1., 5, 5);

  fe->dimension_of_value = 1;
  fe->polynomial_order = 2;
  fe->f = function;  
  fe->size_of_data = sizeof(float) * fe->number_of_points;
  fe->data = (char *)mem_alloc(fe->size_of_data);


  /*---  Aufbauen der Scene-Instanz:  ---*/
  sc = (SCENE *)GRAPE(Scene,"new-instance")("first scene");
  sc->object = (INSTANCE *)fe;
  /*---  Darstellungsmethode:  ---*/
  sc->method_name = "isoline-disp";


  /*  Damit Funktionwerte da sind: ein Schritt:  */
  GRAPE(sc,"start-move")();
  steps = 100;

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();


  /*---  Button zum Starten der Methode:  ---*/
  button_start = (BUTTON *)GRAPE(Button, "new-instance")
		 ("start-move", sc, "start the motion");
  GRAPE(button_start, "set-pref-size") (12., 2.);          
  GRAPE(button_start, "set-color-index") (4);
  GRAPE(mgr, "add-inter") (button_start);


  /*---  Ruler zum Einstellen der Anzahl der Schritte  ---*/
  ruler_steps = (RULER *)GRAPE(Ruler, "new-instance")
                (&steps, "number of steps", RULER_INTEGER);
  GRAPE(mgr, "add-inter") (ruler_steps);


  GRAPE(mgr, "handle") (sc);
}





