
/***********************************************************************/
/*                          isolines2d.c                               */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*    Demoprogramm fuer die Darstellung einer einkomponentigen Funk-   */
/*  tion in 2D z.B. mit Isolinien.                                     */
/*    Als Triangulierung wird ein Quadrat mittels 'explicit' mit 50    */
/*  Dreiecken erzeugt, auf dem als Funktionswert der Abstand vom       */
/*  Nullpunkt definiert wird.                                          */
/*    Da fuer die Isoliniendarstellung standartmaessig die Werte auf   */
/*  den Elemnten linear interpoliert werden, wird die Darstellung ex-  */
/*  akter, wenn der 'refinement-degree' erhoeht wird (was dann laen-   */
/*  gere Darstellungszeitem zur Folge hat).                            */
/*                                        RALPH SCHWOERER (27.07.95)   */
/***********************************************************************/

#include <grape.h>

/*---  Funktionen, die von "explicit" benutzt werden:  ---*/
static float fx(x,y) float x,y; { return(x); }
static float fy(x,y) float x,y; { return(y); }



/*---  Funktion, die fuer jedes Tripel baryzentrischer Koordinaten  ---*/
/*---  den Abstand des Punktes vom Ursprung in *v zurueckgibt:      ---*/

void function(FE2D *fe, int i, float bary[3], float *v)
{ 
  int   j;
  float x = 0. ,y = 0.;
  
  *v =  0.0;
  for(j=0;j<3;j++) {
    x += bary[j] * fe->x[fe->vertex[i][j]];
    y += bary[j] * fe->y[fe->vertex[i][j]];
  }
  *v = g_sqrt(x*x + y*y);
}



main()
{  
  FE2D    *fe;
  MANAGER *mgr;


  printf("\n\n*----------------------------------------------------*");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Choose  'disp meth'  'isoline'                |");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Increase  'refinement-degree'  (in  'opts')   |");
  printf(  "\n|                                                    |");
  printf(  "\n*----------------------------------------------------*\n\n");


  /*---  Automatisches Erzeugen der Fe2d-Instanz: Triangulierung eines  ---*/
  /*---  Quadrates mit den Ecken (-0.5,-0.5) und (0.5,0.5):             ---*/
  fe = (FE2D *)GRAPE(Fe2d, "explicit")
       ("plane", fx, fy, NIL, -0.5, 0.5, -0.5, 0.5, 5, 5);


  /*---  Dimension der darzustellenden Funktion:  ---*/
  fe->dimension_of_value = 1;

  /*---  Grad der Ordnug auf jedem Element (bei Konstruktion dieser  ---*/
  /*---  Funktion ist der Grad eigentlich unendlich):		     ---*/
  fe->polynomial_order = 2;

  /*---  Auswahl der darzustellenden Funktion:  ---*/
  fe->f = function;  


  mgr = (MANAGER *)GRAPE(Manager, "get-stdmgr") ();

  GRAPE(mgr, "handle") (fe);
}





