
/***********************************************************************/
/*                             vect2d.c                                */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*    Demoprogramm fuer die Darstellung einer mehrkomponentigen Funk-  */
/*  tion in 2D mittels Vektorpfeilen.                                  */
/*    Wird das Programm mit dem Kommando 'vect2d' gestartet, wird die  */
/*  Zahl der Komponenten auf 2 festgelegt. Diese Zahl kann durch den   */
/*  Aufruf mit einem Parameter (z.B. 'vect2d  3') gewaehlt werden.     */
/*    Als Triangulierung wird ein Quadrat mittels 'explicit' mit 50    */
/*  Dreiecken erzeugt, auf dem die Komponenten der Funktion folgen-    */
/*  dermassen definiert werden:                                        */
/*    - 1. und 2. Komponente: der x- bzw. y-Wert des Punktes           */
/*    - 3. Komponente: Abstand des Punktes vom Ursprung.               */
/*                                        RALPH SCHWOERER (27.07.95)   */
/***********************************************************************/

#include <grape.h>

/*---  Funktionen, die von "explicit" benutzt werden:  ---*/
static float fx(x,y) float x,y; { return(x);}
static float fy(x,y) float x,y; { return(y);}



/*---  Funktion, die fuer jedes Tripel baryzentrischer Koordinaten die ---*/
/*---  drei Funktionswerte wie oben beschrieben in v zurueckgibt:      ---*/

void function(FE2D *fe, int i, float bary[3], float v[3])
{ 
  int j;
  
  v[0] = v[1] =  0.0;
  for(j=0;j<3;j++) {
    v[0] += bary[j] * fe->x[fe->vertex[i][j]];
    v[1] += bary[j] * fe->y[fe->vertex[i][j]];
  }
  v[2] = g_sqrt(v[0]*v[0] + v[1]*v[1]);
}



main(int argc, char *argv[])
{  
  FE2D    *fe;
  MANAGER *mgr;


  printf("\n\n*----------------------------------------------------*");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Choose  'disp meth'  'vect'                   |");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Decrease  'mesh width'  and  'vect length'    |");
  printf(  "\n|                                                    |");
  printf(  "\n|   (If you want a third component to display then   |");
  printf(  "\n|    start this program with 'vect2d  3')            |");
  printf(  "\n|                                                    |");
  printf(  "\n*----------------------------------------------------*\n\n");


  /*---  Automatisches Erzeugen der Fe2d-Instanz: Triangulierung eines  ---*/
  /*---  Quadrates mit den Ecken (-0.5,-0.5) und (0.5,0.5):             ---*/
  fe = (FE2D *)GRAPE(Fe2d, "explicit")
       ("plane", fx, fy, NIL, -0.5, 0.5, -0.5, 0.5, 5 ,5);


  /*---  Dimension der darzustellenden Funktion:  ---*/
  if (argc > 1)
    sscanf(argv[1],"%i",&fe->dimension_of_value);
  else
    fe->dimension_of_value = 2;

  /*---  Grad der Ordnug:  ---*/
  fe->polynomial_order = 2;

  /*---  Auswahl der darzustellenden Funktion:  ---*/
  fe->f = function;  


  mgr = (MANAGER *)GRAPE(Manager, "get-stdmgr") ();

  GRAPE(mgr, "handle") (fe);
}





