/* @Hdr */

#include <stdio.h>
#include <grape.h>


/* @DOC read_ebsel_fe3d */

/* grape-methode "read-ebsel-3d"
 *  
 * Call              : (FE3D *)GRAPE(Fe3d, "read-ebsel-3d")
 *                                  (inst_name,filename,geoflag)
 *                            
 * Parameters        : CLASS *Fe3d
 *                     char *inst_name, *filename
 *                     int geoflag
 * 
 * Description       : erzeuge Fe3d Instanz aus Ebsel Datei mit druck
 *                     geoflag = 0 ( lies keine geometrie ein)
 *                     geoflag = 1 ( lies geometrie mit ein  )
 *                              
 * Keywords          : read-ebsel fe3d
 *
 * Author            : alfred schmidt
 *
 * Copyright (c) 1990     by Universitaet Bonn
 *                           Institut fuer Angewandte Mathematik
 *                           Sonderforschungsbereich 256
 *                           D-5300 Bonn 1
 *
 * history           : 06.03.90 erstellt
 *
 */



/* @SRC f3d_rd_ebsel.c */

typedef float FLOAT4[4];
typedef float FLOAT3[3];
typedef int INT6[6];

void gaga(fe,k,bary,vec)

FE3D *fe;
int k;
float bary[4],vec[];
{
  int i,j,jj,j_count;
  VEC3 p;
  INT6 *tetrae;
    
  tetrae = (INT6 *)(&fe->vertex[fe->number_of_elements][0]);

  if(fe->dimension_of_value == 3) 
    for(i=0;i<3;i++) {
      for(vec[i] = 0.0, j = 0;j<4;j++) {
	vec[i] += bary[j] * (2.*bary[j]-1.) * 
	  ((FLOAT4 *)fe->data)[fe->vertex[k][j]][i];
      }
      for(j=j_count=0;j<3;j++){
	for(jj=j+1;jj<4;jj++) {
	  vec[i] += 4 * bary[j] * bary[jj] * 
	    ((FLOAT4 *)fe->data)[tetrae[k][j_count]][i];
	  j_count++;
	}
      }
      
   }
  else
    for(vec[0] = 0, j = 0;j<4;j++) {
      vec[0] += bary[j] * ((FLOAT4 *)fe->data)[fe->vertex[k][j]][3];
    }
}


FE3D *fe3d_read_ebsel_3d(inst_name, filename, geoflag)
     char *inst_name, *filename;
     int geoflag;

{
  FILE *geom_file;
  int nnode,ntetra;
  float *x, *y, *z;
  INT4 *tetrap, *tetrad, int4;
  INT6 *tetrae, int6;
  FLOAT3 real3;
  FLOAT4 real4;
  CLASS *class;
  FE3D *fe;
  int i,j;
  float muell[1];
  int geo,pressure;

  fe = NIL;
  class = (CLASS *)START_METHOD(G_CLASS);
  ASSURE(class,
	 "read-ebsel : can't manage instances",
	 END_METHOD(NIL));

  /* read in the triangulation.
     x,y,z become the vertex coorinates,
     fe->data become the vertex values.

     tetrap[][4] become the vertex numbers for each tetraeders 
     tetrae[][6] become the vertex numbers for a mid point of an edge 
     tetrad[][4] become the neighbour numbers for each tetraeders */

  fe = (FE3D *)GRAPE(class,"new-instance")(inst_name);
  ASSURE(fe,
	 "read-ebsel-3d : cannot create Fe3d instance",
	 END_METHOD(NIL));
  
  ASSURE(((geom_file=fopen(filename,"r")) != NULL),
	 "read-ebsel-3d: cannot open input file",
	 END_METHOD(NIL));

  fread(&geo,1,sizeof(int),geom_file);

  ASSURE((!geoflag || geo),
	 "read-ebsel-3d : can't read geometry\n",
	 END_METHOD(NIL));
	 



  fread(&nnode,1,sizeof(int),geom_file);
  fread(&ntetra,1,sizeof(int),geom_file);

  ASSURE( ((ntetra>0) && (nnode>0)),
	 "empty tetrangulation",
	 END_METHOD(NIL) );
  
  fe->number_of_points = nnode;
  fe->max_number_of_points = nnode;
  fe->number_of_elements = ntetra;
  fe->max_number_of_elements = ntetra;

  if(geoflag) {
    fe->x = x = (float *)mem_alloc(nnode*sizeof(float));
    fe->y = y = (float *)mem_alloc(nnode*sizeof(float));
    fe->z = z = (float *)mem_alloc(nnode*sizeof(float));
    fe->vertex = tetrap = 
      (INT4 *)mem_alloc(ntetra*sizeof(INT4)+ntetra*sizeof(INT6));
    tetrae = (INT6 *)(&fe->vertex[fe->number_of_elements][0]);
    fe->neighbour = tetrad = (INT4 *)mem_alloc(ntetra*sizeof(INT4));
    ASSURE( x&&y&&z&&tetrap&&tetrad,
	 "read-ebsel-3d : can not allocate memory for geometry",
	 END_METHOD(NIL));  
  }
   
  fe->data = (char *)mem_alloc(nnode*sizeof(FLOAT4));
  fe->size_of_data = nnode*sizeof(FLOAT4);
  
  ASSURE( fe->data, "read-ebsel-3d : can not allocate memory for fe->data",
	 END_METHOD(NIL));  




  for (i=0;i<nnode;i++) {

       if(geo)  fread(&real3,1,sizeof(FLOAT3),geom_file);
       if(geoflag) { x[i] = real3[0]; y[i] = real3[1]; 
                     z[i] = real3[2];}

       fread(&real4,1,sizeof(FLOAT4),geom_file);

       for(j=0;j<4;j++) ((FLOAT4 *)fe->data)[i][j] = real4[j];


  }



  if(geoflag) {
    for (i=0;i<ntetra;i++) {
        fread(&int4,1,sizeof(INT4),geom_file);
        for(j=0;j<4;j++) tetrap[i][j] = int4[j];

        fread(&int6,1,sizeof(INT6),geom_file);
        for(j=0;j<6;j++) tetrae[i][j] = int6[j];
    }


    fread(tetrad,ntetra,sizeof(INT4),geom_file);


    }
  



  if(geoflag) {
    fe->vertex = tetrap;
    fe->neighbour = tetrad;
  }
  
  fe->f = gaga;
  fe->dimension_of_value = 3;
  fe->polynomial_order = 2;
  
  fclose(geom_file);
  END_METHOD(fe);
  
}
