/* @Hdr */

#include <stdio.h>
#include <grape.h>


/* @DOC read_ebsel_fe3d */

/* grape-methode "read-ebsel-3d"
 *  
 * Call              : (FE3D *)GRAPE(Fe3d, "read-ebsel-3d")
 *                                  (inst_name,filename,geoflag)
 *                            
 * Parameters        : CLASS *Fe3d
 *                     char *inst_name, *filename
 *                     int geoflag
 * 
 * Description       : erzeuge Fe3d Instanz aus Ebsel Datei mit druck
 *                     geoflag = 0 ( lese keine geometrie ein)
 *                     geoflag = 1 ( lese geometrie mit ein  )
 *                              
 * Keywords          : read-ebsel fe3d
 *
 * Author            : alfred schmidt
 *
 * Copyright (c) 1990     by Universitaet Bonn
 *                           Institut fuer Angewandte Mathematik
 *                           Sonderforschungsbereich 256
 *                           D-5300 Bonn 1
 *
 * history           : 06.03.90 erstellt
 *
 */



/* @SRC f3d_rd_ebsel.c */

typedef float FLOAT4[4];
typedef int INT6[6];

void gaga(fe,k,bary,vec)

FE3D *fe;
int k;
float bary[4],vec[];
{
  int i,j,jj,j_count;
  VEC3 p;
  INT6 *tetrae;
    
  tetrae = (INT6 *)(&fe->vertex[fe->number_of_elements][0]);

  if(fe->dimension_of_value == 3) 
    for(i=0;i<3;i++) {
      for(vec[i] = 0.0, j = 0;j<4;j++) {
	vec[i] += bary[j] * (2.*bary[j]-1.) * 
	  ((FLOAT4 *)fe->data)[fe->vertex[k][j]][i];
      }
      for(j=j_count=0;j<3;j++){
	for(jj=j+1;jj<4;jj++) {
	  vec[i] += 4 * bary[j] * bary[jj] * 
	    ((FLOAT4 *)fe->data)[tetrae[k][j_count]][i];
	  j_count++;
	}
      }
      
   }
  else
    for(vec[0] = 0, j = 0;j<4;j++) {
      vec[0] += bary[j] * ((FLOAT4 *)fe->data)[fe->vertex[k][j]][3];
    }
}


FE3D *fe3d_read_ebsel_3d(inst_name, filename, geoflag)
     char *inst_name, *filename;
     int geoflag;

{
  FILE *geom_file;
  int nnode,ntetra;
  float *x, *y, *z;
  INT4 *tetrap, *tetrad;
  INT6 *tetrae;
  CLASS *class;
  FE3D *fe;
  int i;
  float muell[1];
  int geo,pressure;

  fe = NIL;
  class = (CLASS *)START_METHOD(G_CLASS);
  ASSURE(class,
	 "read-ebsel : can't manage instances",
	 END_METHOD(NIL));

  /* read in the triangulation.
     x,y,z become the vertex coorinates,
     fe->data become the vertex values.

     tetrap[][4] become the vertex numbers for each tetraeders 
     tetrae[][6] become the vertex numbers for a mid point of an edge 
     tetrad[][4] become the neighbour numbers for each tetraeders */

  fe = (FE3D *)GRAPE(class,"new-instance")(inst_name);
  ASSURE(fe,
	 "read-ebsel-3d : cannot create Fe3d instance",
	 END_METHOD(NIL));
  
  ASSURE(((geom_file=fopen(filename,"r")) != NULL),
	 "read-ebsel-3d: cannot open input file",
	 END_METHOD(NIL));

  fscanf(geom_file,"%d",&geo);
  ASSURE((!geoflag || geo),
	 "read-ebsel-3d : can't read geometry\n",
	 END_METHOD(NIL));
	 
  fscanf(geom_file,"%d",&pressure);
  fscanf(geom_file,"%d%d",&nnode,&ntetra);
  ASSURE( ((ntetra>0) && (nnode>0)),
	 "empty tetrangulation",
	 END_METHOD(NIL) );
  
  fe->number_of_points = nnode;
  fe->max_number_of_points = nnode;
  fe->number_of_elements = ntetra;
  fe->max_number_of_elements = ntetra;

  if(geoflag) {
    fe->x = x = (float *)mem_alloc(nnode*sizeof(float));
    fe->y = y = (float *)mem_alloc(nnode*sizeof(float));
    fe->z = z = (float *)mem_alloc(nnode*sizeof(float));
    fe->vertex = tetrap = 
      (INT4 *)mem_alloc(ntetra*sizeof(INT4)+ntetra*sizeof(INT6));
    tetrae = (INT6 *)(&fe->vertex[fe->number_of_elements][0]);
    fe->neighbour = tetrad = (INT4 *)mem_alloc(ntetra*sizeof(INT4));
  }
   
  fe->data = (char *)mem_alloc(nnode*sizeof(FLOAT4));
  fe->size_of_data = nnode*sizeof(FLOAT4);
  
  ASSURE( ((!geoflag || (x&&y&&z&&tetrap&&tetrad)) && fe->data),
	 "read-ebsel-3d : can not allocate memory",
	 END_METHOD(NIL));
  for (i=0;i<nnode;i++) {
    if(geo) {  
      if(geoflag) {
	fscanf(geom_file,"%f%f%f%f%f%f",
	       &(x[i]),&(y[i]),&(z[i]),
	       &(((FLOAT4 *)fe->data)[i][0]),&(((FLOAT4 *)fe->data)[i][1]),
	       &(((FLOAT4 *)fe->data)[i][2]));
	if(pressure) fscanf(geom_file,"%f",&(((FLOAT4 *)fe->data)[i][3]));
	else ((FLOAT4 *)fe->data)[i][3] = 0.0;
	((FLOAT4 *)fe->data)[i][3] *= 10000. ;
      }
      else {
	fscanf(geom_file,"%f%f%f%f%f%f",
	       muell,muell,muell,
	       &(((FLOAT4 *)fe->data)[i][0]),&(((FLOAT4 *)fe->data)[i][1]),
	       &(((FLOAT4 *)fe->data)[i][2]));
	if(pressure) fscanf(geom_file,"%f",&(((FLOAT4 *)fe->data)[i][3]));
	else ((FLOAT4 *)fe->data)[i][3] = 0.0;
	((FLOAT4 *)fe->data)[i][3] *= 10000. ;
      }
    }
  }
  if(geo) {
    if(geoflag) {
      for (i=0;i<ntetra;i++) {
	fscanf(geom_file,"%d%d%d%d",
	       &(tetrap[i][0]),&(tetrap[i][1]),&(tetrap[i][2]),
	       &(tetrap[i][3]));
	fscanf(geom_file,"%d%d%d%d%d%d",
	       &(tetrae[i][0]),&(tetrae[i][1]),&(tetrae[i][2]),
	       &(tetrae[i][3]),&(tetrae[i][4]),&(tetrae[i][5]));
      }
      for (i=0;i<ntetra;i++) {
	fscanf(geom_file,"%d%d%d%d",
	       &(tetrad[i][0]),&(tetrad[i][1]),&(tetrad[i][2]),
	       &(tetrad[i][3]));
      }
    }
  }
  
  if(geoflag) {
    fe->vertex = tetrap;
    fe->neighbour = tetrad;
  }
  
  fe->f = gaga;
  fe->dimension_of_value = 3;
  fe->polynomial_order = 2;
  
  fclose(geom_file);
  END_METHOD(fe);
  
}






