/*******************************************/
/*  headerfile fuer finite element numerik */
/*                in 2d                    */
/*******************************************/


#define N 3  /* dimension des problems                       */
#define NB 4 /* anzahl der barycentrischen koordinaten       */

#define NEUMANN -2      /* neumann rand kanten eintrag in tr->neighbour  */ 
#define DIRICHLET -1    /* dirichlet rand kanten in tr->neighbour        */

#define ITMAX 10     /* maximale iterationen des cg verfahrens        */
#define INFINITY  1e20  /* wert des funktional undefininiert             */
#define EPS 1e-4        /* fehlerschranke fur residuumquadrat im cg      */

#define DELTA 1e-3      /* differenzenquotientennenner                   */

typedef double precision;    /* allgemeine rechengenauigkeit  */
                             /* koordinaten bleiben float     */
#ifndef precision
  typedef precision VECT[N];
#endif 

typedef int INTNB[NB];

typedef struct parameter {
  float homotopy;
  int   itmax;
  float eps;
} PARAMETER;

typedef struct additional{
  precision det;
  precision qcof;
  precision qa;
} ADDITIONAL;

#include<grape.h>

#ifdef GRAPE 

#define FE FE3D 
#define get_memory  mem_alloc                   /* oder malloc           */
#define free_memory(a,b) mem_free((a),(b))      /* oder free((b))        */

#else 

#define get_memory  malloc
#define free_memory(a,b) mem_free((a)) 

typedef struct fe {
  int number_of_points;
  int max_number_of_points;
  float *x, *y;
  int number_of_elements;
  int max_number_of_elements;
  INTNB *vertex;    
  INTNB *neighbour;
  int dimension_of_value; /* 1==scalar,  >1 => vector function */
  int polynomial_order;   /* 1==linear, 2==quadratic,... elements */
  void (*f)();            /* void *f( FE3D fe;
			     int triangle;
			     VEC4 barycentric_coordinates;
			     float value[dim]; )          */
  int size_of_data;      /* bereich um eigene daten zur loesung          */   
  char *data;            /* abzulegen und dessen groesse                 */
} FE;

#endif

typedef struct refmap {
  precision p0[N],a[N][N];  /* refabb. ist p0 + a * x ( x aus ref-element) */
  precision meas;           /* volumen des elements                        */
  precision gradb[NB][N];   /* gradb[i] = gradient von bary-koord[i]      */
} REFMAP;

typedef struct imagemap {
  precision det,qcof,qa;
  precision cof[N][N],a[N][N];
  precision dqa[N][N],dqcof[N][N];
} IMAGEMAP;

typedef struct constants {
  precision a,b,c,d,e;
} CONSTANTS;








