
/***********************************************************************/
/*                            isolines3d.c                             */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*    Demoprogramm fuer die Darstellung einer einkomponentigen Funk-   */
/*  tion in 3D z.B. mit Isolinien.                                     */
/*    Als Triangulierung wird ein Quadrat mittels 'explicit' mit 50    */
/*  Dreiecken erzeugt, aus dem dann mit 'shift' ein Wuerfel gemacht    */
/*  wird. Als Funktionswert wird der Abstand vom Nullpunkt definiert.  */
/*                                        RALPH SCHWOERER (27.07.95)   */
/***********************************************************************/

#include <grape.h>

/*---  Funktionen, die von "explicit" benutzt werden:  ---*/
static float fx(x,y) float x,y; { return(x);}
static float fy(x,y) float x,y; { return(y);}



/*---  Funktion, die fuer jedes Tripel baryzentrischer Koordinaten  ---*/
/*---  den Abstand des Punktes vom Ursprung in *v zurueckgibt:      ---*/

void function(FE3D *fe, int i, float bary[3], float *v)
{ 
  int   j;
  float x = 0. ,y = 0., z = 0.;
  
  *v =  0.0;
  for(j=0;j<4;j++) {
    x += bary[j] * fe->x[fe->vertex[i][j]];
    y += bary[j] * fe->y[fe->vertex[i][j]];
    z += bary[j] * fe->z[fe->vertex[i][j]];
  }
  *v = g_sqrt(x*x + y*y + z*z);
}



main()
{  
  FE2D    *fe2d;
  FE3D    *fe3d;
  MANAGER *mgr;
  int     i, j;
 
  printf("\n\n*----------------------------------------------------*");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Rotate the body in the  'trans' -menu         |");
  printf(  "\n|                                                    |");
  printf(  "\n|   +  Choose one of the  'disp meth's:              |");
  printf(  "\n|      'clip-isoline' ,  'clip-iso-select' ,         |");
  printf(  "\n|      'bnd-isoline' , or  'bnd-iso-select'          |");
  printf(  "\n|                                                    |");
  printf(  "\n*----------------------------------------------------*\n\n");


  /*---  Automatisches erzeugen der Fe2d-Instanz: Triangulierung eines  ---*/
  /*---  Quadrates mit den Ecken (-0.5,-0.5) und (0.5,0.5):             ---*/
  fe2d = (FE2D *)GRAPE(Fe2d, "explicit")
	 ("Quadrat", fx, fy, NIL, -0.5, 0.5, -0.5, 0.5, 5, 5);

  /*---  Erzeugen der Fe3d-Instanz durch "shiften" des Quadrates  ---*/
  fe3d = (FE3D *)GRAPE(fe2d, "shift") ("Wuerfel", 5, 0.2);

  /*---  Setzen der Nachbarschaftsbeziehungen (fuer "bnd-isoline"):  ---*/
  GRAPE(fe3d,"identify-nbs-send")();


  /*---  Dimension der darzustellenden Funktion:  ---*/
  fe3d->dimension_of_value = 1;

  /*---  Grad der Ordnug auf jedem Element (bei Konstruktion dieser  ---*/
  /*---  Funktion ist der Grad eigentlich unendlich):		     ---*/
  fe3d->polynomial_order = 2;

  /*---  Auswahl der darzustellenden Funktion:  ---*/
  fe3d->f = function;  


  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();

  GRAPE(mgr,"handle")(fe3d);
}





