/**********************************************************/
/*                                                        */
/*   this is a demo for the working with the included     */
/*   finite element structures                            */
/*   (i.e. on how to convert finite difference data in    */
/*    finite element convert using GRAPE)                 */
/*                                                        */
/*   the underlying function is                           */
/*                                                        */
/*        f = (1/(x-ai)^2)                                */
/*                        i=1..3                          */
/*                                                        */
/*            ai = (xi,yi,zi)                             */
/*                                                        */
/*   The option menu allows interactive changing of ai.   */
/*                                                        */
/*                                                        */
/**********************************************************/

#include <grape.h>

float a,b,c,d1,d2,d3;

/*************************************************/
/*  finite element funktion joined to fe3d later */ 
/*************************************************/

static void calc_value(fe,k,bary,vec)

FE3D *fe;
int k;
float bary[4],vec[];

{

  int j,i,dim;
  float x,y,z;
  float norm;

  x = y = z = 0;
  for(j = 0;j<4;j++) {
    x += bary[j] * fe->x[fe->vertex[k][j]];
    y += bary[j] * fe->y[fe->vertex[k][j]];
    z += bary[j] * fe->z[fe->vertex[k][j]];
  }
  vec[0] = 1.-a*(x-d1)*(x-d1)+b*(y-d2)*(y-d2)+c*(z-d3)*(z-d3);

  return;
}


main()

{

  GRAPHICDEVICE *dev,*ps_dev;
  FE3D *fe;
  MANAGER *mgr;
  RULER *rula,*rulb,*rulc;
  RULER *ruld1,*ruld2,*ruld3;
  char name[30];
  float *x,*y,*z;
  int xn,yn,zn;
  int i;
  VEC3 *vect;

 
  x = NIL;y = NIL;z = NIL;

  printf("feinheit (z.b. : 10 10 10 ): ");scanf("%d %d %d",&xn,&yn,&zn);
  printf("\n");

  fe = (FE3D *)GRAPE(Fe3d,"^make")(x,y,z,xn,yn,zn,"");
  ASSURE(fe,"can't build fe3d instance",exit(1));

  fe->dimension_of_value = 1;
  fe->polynomial_order = 1;
  fe->f = calc_value;

/* einige interaktive methoden werden als  */
/* button direkt zur verfuegung gestellt   */

/* die positionen der potentialsenken werden an schieber gehangen */  

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();
  rula = (RULER *)GRAPE(Ruler,"new-instance")(&a,"a",RULER_FLOAT);
  GRAPE(mgr,"add-inter")(rula);
  rulb = (RULER *)GRAPE(Ruler,"new-instance")(&b,"b",RULER_FLOAT);
  GRAPE(mgr,"add-inter")(rulb);
  rulc = (RULER *)GRAPE(Ruler,"new-instance")(&c,"c",RULER_FLOAT);
  GRAPE(mgr,"add-inter")(rulc);
  ruld1 = (RULER *)GRAPE(Ruler,"new-instance")(&d1,"d1",RULER_FLOAT);
  GRAPE(mgr,"add-inter")(ruld1);
  ruld2 = (RULER *)GRAPE(Ruler,"new-instance")(&d2,"d2",RULER_FLOAT);
  GRAPE(mgr,"add-inter")(ruld2);
  ruld3 = (RULER *)GRAPE(Ruler,"new-instance")(&d3,"d3",RULER_FLOAT);
  GRAPE(mgr,"add-inter")(ruld3);
    
/* das interaktive paket wird aufgerufen  */


  GRAPE(mgr,"handle")(fe);


}





