
/****************************************************************/
/*         definitions and structures                           */
/****************************************************************/

#include <grape.h>
#include <stdlib.h>

#define GAMMA 1.4       /* air gas constant */

#define sqr(A) ((A)*(A))

typedef double DOUVEC3[3];
typedef float FLOAT32[3][2];

typedef struct fvsys {

        /* SYSTEM ONLY */
        CLASS *class;
        char *name;
        INSTANCE *next;
        unsigned int refcount;
        /* USER */
        int number_of_points;
        int max_number_of_points;
        float *x, *y, *z;
        int number_of_elements;
        int max_number_of_elements;
        INT3 *vertex;
        INT3 *neighbour;

        /* SUBCLASS */
        int dimension_of_value;
        int polynomial_order;
        void (*f)();            /* void *f( FE2D fe;
                                            int triangle;
                                            VEC3 barycentric_coordinates;
                                            float value[dim]; )        */
/* optional ********************************************************** */
        int size_of_data;
        char *data;
/* ******************************************************************* */
        REFINE2D *refine;
        int number_of_bnd_functions; /* (max) no of different boundaries */
        int max_number_of_bnd_functions;
        BND_FUNCTION *bnd_function;   /* bnd_function[-neighbour[i][j]]  */
                                      /*  ->g(x,bnd_function->par) = 0.0 */
                                      /* ist der rand mit VEC3 x         */
          /* Functions for interpolation and restriction
           * calls: interpol(ADAPT2D *tr,
           *                 int oldtriangle, int oldpoint1, int oldpoint2,
           *                 int newpoint, int newtriang1, int newtriang2)
           * oldpoint1 and oldpoint2 are the global points of the divided edge
           * newpoint is the global number of the new (added) point)
           *
           *        restrict(ADAPT2D *tr,
           *                 int deletriang1, int deletriang2, int deletpoint,
           *                 int newtriang, int makropoint1, int makropoint2)
           * deletpoint is the global number of the deleted point
           * makropoint1 and makropoint2 are the global points of the
           * coarsened edge in the new triangle
           *
           * while coarsening, gaps in the triangulation-arrays have to be
           * filled by moving elements or points with:
           *         element_compress(ADAPT2D *tr, int old_element_number,
           *                          int new_element_number)
           *         point_compress(ADAPT2D *tr, int old_point_number,
           *                          int new_point_number)
           */
        void (*interpol)(), (*restrict)();
        void (*element_compress)(), (*point_compress)();

  float *tr_area;               /* areas of the elements                */
  VEC2  *tr_center;             /* centers of gravity                   */
  float maxdiam;                /* max lengh of edges                   */
  float minarea;                /* minimal area of elements             */
  float xdelta;                 /* min lengh of edges                   */
  float cfl;                    /* max CFL-number                       */
  FLOAT32 *tr_normal;           /* normals to edges of elements         */
  double **u,**u_dt;
  float time;
  int step;                     /* number of calculated timesteps       */
} FVSYS;

extern CLASS *FvSys;

extern int POL_ORD;
extern void (*ISOFUNC)();
extern int DOV;

extern int NUMBER_OF_KOMPS;
extern int KOMP;

extern FVSYS *load(int const, char *const, FVSYS *);
extern double **malloc_data_mem (int const);
extern double **free_data_mem(double **, int const);
extern void set_tr_center(FVSYS *const);

extern RULER *ruler_switch_komp(void);
extern TIMESCENE *fv_switch_isofunc(void);
 
extern void isofunc0(FVSYS *, int , float[3], float *);
extern void isofunc1(FVSYS *, int , float[3], float *);
extern void isofunc_v(FVSYS *, int , float[3], float *);

extern FVSYS *fvsys_interpol(FVSYS *, FVSYS *, double );
extern void mess_out(void);

/* basic subroutines for Euler Equations */
extern double make_primitive(double *const ,const int );
extern double make_conservative(double *const,const int );
extern double machnumber(double *const);
extern double entropy(double *const);

