
#include "ffs.h"

/********************************************************************/
/*          basis routines for Euler-Equations in 2d                */
/********************************************************************/

double make_primitive(double *const u, const int komp)   
				/* U(conservativ) --> V(primitiv) */
				/* komponent komp of vector */
{

  switch (komp) {

  case 0:
    return (u[0]);
  case 1:
    return (u[1] / u[0]);
  case 2:
    return (u[2] / u[0]);
  case 3:
    return ((GAMMA-1.0) * (u[3]-0.5/u[0]*(u[1]*u[1]+u[2]*u[2])));
  default:
    if (komp < NUMBER_OF_KOMPS)
      return (u[komp] / u[0]);
    else
      return (0.0);
  }

}


double make_conservative(double *const v, const int komp) 
				/* V(primitiv) --> U(conservativ) */
				/* komponent komp of vector */
{

  switch (komp) {

  case 0:
    return (v[0]);
  case 1:
    return (v[1] * v[0]);
  case 2:
    return (v[2] * v[0]);
  case 3:
    return (v[3]/(GAMMA-1.0) + 0.5*v[0]*(v[1]*v[1]+v[2]*v[2]));
  default:
    if (komp < NUMBER_OF_KOMPS)
      return (v[komp] * v[0]);
    else
      return (0.0);

  }

}


double machnumber(double *const u)	/* u conservative */

{

  double machnb;

  machnb = sqrt((u[1]*u[1] + u[2]*u[2]) / (GAMMA*u[0]*make_primitive(u,3)));

  return(machnb);

}


double entropy(double *const u)		/* u conservative */

{
  const double S_NULL = 0.0;	/* two constants depending on the gas */
  const double C_NUE  = 1.0;

  double entropy;

  entropy = S_NULL + C_NUE * log(make_primitive(u,3) /
					(pow(u[0],GAMMA)*(GAMMA -1.0)));

  return(entropy);

}


double temperature(double *const u)	/* u conservative */

{
  const double R = 1.0;		/* Boltzmann constant */
  const double M = 1.0;		/* weight of molecules */

  double temperature;

  temperature = M/R * make_primitive(u,3)/u[0];

  return(temperature);

}
