
#include "ffs.h"

FVSYS *fvsys_interpol(FVSYS *fe1, FVSYS *fe2, double mu)

{ 

  FVSYS *fe;
  int i,j;
  static double **u = NULL;
  static int ucount = 0;

  ASSURE(fe = (FVSYS *)START_METHOD(G_INSTANCE),
	 "interpol : neither class nor instance found",
	 END_METHOD(NIL));
  if(fe2 != NIL)
    ASSURE((fe1->class == fe2->class),
	   "interpol : objects of different classes",
	   END_METHOD(NIL));
  ASSURE(fe = (FVSYS *)GRAPE(fe,"^interpol")(fe1,fe2,mu),
	 "interpol : can't interpol fe2d part of fvsys",
	 END_METHOD(NIL));

  fe->refine = fe1->refine;

  fe->tr_area = fe1->tr_area;
  fe->tr_center = fe1->tr_center;
  fe->maxdiam = fe1->maxdiam;
  fe->minarea = fe1->minarea;
  fe->xdelta = fe1->xdelta;
  fe->cfl = fe1->cfl;
  fe->tr_normal = fe1->tr_normal;
 
  if (ucount < fe->number_of_elements) {
    if (u != NULL)
      u = free_data_mem(u,ucount);
    u = malloc_data_mem(fe->number_of_elements);
    ASSURE(u,"interpol : can't allocate memory",return(NULL));
    ucount = fe->number_of_elements;
  }

  if (fe2 != NULL) {
    for (i=0 ; i<fe->number_of_elements ; i++)
      for (j=0 ; j<NUMBER_OF_KOMPS ; j++)
        u[i][j] = (1.0 - mu) * fe1->u[i][j] + mu * fe2->u[i][j];  
    fe->time = (1.0 - mu) * fe1->time + mu * fe2->time;
    fe->step = (int) ((1.0 - mu) * fe1->step + mu * fe2->step);
  }
  else {
    for (i=0 ; i<fe->number_of_elements ; i++)
      for (j=0 ; j<NUMBER_OF_KOMPS ; j++)
        u[i][j] = fe1->u[i][j];
    fe->time = fe1->time;
    fe->step = fe1->step;
  }

  fe->u = u;

  END_METHOD(fe);

}
