
#include "ffs.h"

/****************************************************************/
/*        output                                                */
/****************************************************************/

void isofunc0(FVSYS *fv, int nr, float bary[3], float *result)
                                        /* output with pol_ord 0 (isoplanes) */
{

    if (KOMP < NUMBER_OF_KOMPS)
      *result = make_primitive(fv->u[nr],KOMP);
    else if (KOMP == NUMBER_OF_KOMPS)       /* output of machnumber */
      *result = machnumber(fv->u[nr]);
    else if (KOMP == (NUMBER_OF_KOMPS + 1)) /* output of temperature */
      *result = temperature(fv->u[nr]);
    else if (KOMP == (NUMBER_OF_KOMPS + 2)) /* output of entropy */
      *result = entropy(fv->u[nr]);

}


void isofunc1(FVSYS *fv, int nr, float bary[3], float *result)
                                        /* output with pol_ord 1 (isolines) */
{

  int i,j,k;
                                /* for interpolated output : */
                                /*  node values due to Frink */
  static double *weight = NULL;
  static float *zout = NULL;
  static int gr_weight = 0;
  double r;

  if (nr == 0) {

    if (gr_weight < fv->max_number_of_points) {
      if (gr_weight > 0) {
        mem_free(weight,sizeof(double)*gr_weight);
        mem_free(zout,sizeof(float)*gr_weight);
      }
      weight = (double *)mem_alloc(sizeof(double)*fv->max_number_of_points);
      zout = (float *)mem_alloc(sizeof(float)*fv->max_number_of_points);
      ASSURE(weight&&zout,"isofunc1 : can't allocate memory",return);
      gr_weight = fv->max_number_of_points;
    }

    for (i=0 ; i<fv->number_of_points ; i++) {
      zout[i] = 0.0f;
      weight[i] = 0.0;
    }
    for (j=0 ; j<fv->number_of_elements ; j++)
      for (k=0 ; k<3 ; k++) {
        r = sqrt( sqr( fv->tr_center[j][0] - fv->x[fv->vertex[j][k]] ) +
                  sqr( fv->tr_center[j][1] - fv->y[fv->vertex[j][k]] )  );
        if (KOMP < NUMBER_OF_KOMPS)
          zout[fv->vertex[j][k]] += make_primitive(fv->u[j],KOMP) / r;
        else if (KOMP == NUMBER_OF_KOMPS)           /* output of machnumber */
          zout[fv->vertex[j][k]] += machnumber(fv->u[j]) / r;
        else if (KOMP == (NUMBER_OF_KOMPS + 1))     /* output of temperature */
          zout[fv->vertex[j][k]] += temperature(fv->u[j]) / r;
        else if (KOMP == (NUMBER_OF_KOMPS + 2))     /* output of entropy */
          zout[fv->vertex[j][k]] += entropy(fv->u[j]) / r;

        weight[fv->vertex[j][k]] += 1.0 / r;
      }

    for (i=0 ; i<fv->number_of_points ; i++)
      zout[i] /= weight[i];

  }

  *result = 0;
  for (i=0 ; i<3 ; i++)
    *result += zout[fv->vertex[nr][i]] * bary[i];

}


void isofunc_v(FVSYS *fv, int nr, float bary[3], float *result)
                                        /* velocity with pol_ord 0 */

{
    
    result[0] = fv->u[nr][1];
    result[1] = fv->u[nr][2];

}


void set_tr_center(FVSYS * const fv)    /* calculates centers of gravity */

{

  int i;

  if (fv->tr_center == NULL)
    fv->tr_center = (VEC2 *)
                mem_alloc(sizeof(VEC2) * fv->max_number_of_elements);
  ASSURE(fv->tr_center,"fv-set : can't allocate memory",return);

  for (i=0 ; i<fv->number_of_elements ; i++) {
    fv->tr_center[i][0] = 1./3. * (fv->x[fv->vertex[i][0]] +
                                   fv->x[fv->vertex[i][1]] +
                                   fv->x[fv->vertex[i][2]]  );
    fv->tr_center[i][1] = 1./3. * (fv->y[fv->vertex[i][0]] +
                                   fv->y[fv->vertex[i][1]] +
                                   fv->y[fv->vertex[i][2]]  );
  }

}


RULER *ruler_switch_komp(void)       /* switches to next output component */

{

    RULER *ruler;

    ruler = (RULER *)START_METHOD(G_INSTANCE);
    ASSURE(ruler,"no ruler",END_METHOD(NULL));

    mess_out();

  END_METHOD(ruler)

}


TIMESCENE *fv_switch_isofunc(void)       /* switches ISOFUNC */

{

  TIMESCENE *sc;
  TIMESTEP *tst;
  INSTANCE *object;
  FVSYS *fv_pre,*fv_post;
  MANAGER *mgr;

  sc = (TIMESCENE *)START_METHOD(G_INSTANCE);

  ASSURE(sc,"2dfvsy-switch-isofunc : can't manage classes",END_METHOD(sc));

  object = sc->object;
  ASSURE(g_has_superclass(object,FvSys),
         "2dfvsy-switch-isofunc : current object is not of class FvSys",
         END_METHOD(sc));

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();

  tst = (TIMESTEP *)sc->dynamic;
  while (tst->post_step)                 /* set tst on last step */
    tst = tst->post_step;

  if (ISOFUNC == isofunc0) {
    ISOFUNC = isofunc1;
    POL_ORD = 1;
    sc->time = sc->time + 0.0000001;
    DOV = 1;
  }
  else if (ISOFUNC == isofunc1) {
    ISOFUNC = isofunc_v;
    POL_ORD = 0;
    sc->time = sc->time + 0.0000001;
    DOV = 2;
  }
  else if (ISOFUNC == isofunc_v) {
    ISOFUNC = isofunc0;
    POL_ORD = 0;
    sc->time = sc->time - 0.0000002;
    DOV = 1;
  }

  while (tst) {
    if ((g_has_superclass(tst->pre_object,FvSys)) &&
        (g_has_superclass(tst->post_object,FvSys))   ) {
      fv_pre = (FVSYS *)tst->pre_object;
      fv_post = (FVSYS *)tst->post_object;
      fv_pre->f = ISOFUNC;
      fv_post->f = ISOFUNC;
      fv_pre->polynomial_order = POL_ORD;
      fv_post->polynomial_order = POL_ORD;
      fv_pre->dimension_of_value = DOV;
      fv_post->dimension_of_value = DOV;
    }
    tst = tst->pre_step;
  }

  mess_out();

  END_METHOD(sc)

}


void mess_out(void)  /* output of parameters using set-message */

{

  MANAGER *mgr;
  char mess1[60],mess2[60],mess3[60];

  switch (KOMP) {

    case 0:
      sprintf(mess1,"output komponent: density");
      break;
    case 1:
      sprintf(mess1,"output komponent: x-velocity");
      break;
    case 2:
      sprintf(mess1,"output komponent: y-velocity");
      break;
    case 3:
      sprintf(mess1,"output komponent: pressure");
      break;
    case 4:
      sprintf(mess1,"output komponent: machnumber");
      break;
    case 5:
      sprintf(mess1,"output komponent: temperature");
      break;
    case 6:
      sprintf(mess1,"output komponent: entropy");
      break;
  }

  if (ISOFUNC == isofunc0) {
    sprintf(mess2,"isoline function: piecewise constant"); 
    sprintf(mess3,"");
  }
  else if (ISOFUNC == isofunc1) {
    sprintf(mess2,"isoline function: piecewise linear"); 
    sprintf(mess3,"                  (interpolated)"); 
  }
  else if (ISOFUNC == isofunc_v) {
    sprintf(mess2,"isoline function: velocity vector"); 
    sprintf(mess3,"");
  }

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();
  GRAPE(mgr,"set-message")(mess1,mess2,mess3);

}

