
#include "ffs.h"

/****************************************************************/
/*       load                                                   */
/****************************************************************/

extern int errno;

void write_triang(FVSYS *const fv, char *name)
                        /* writes triangulation in ASCII-file "name" */

{
  FILE *hdl;
  int i,j,k;

  hdl = fopen(name,"w");
  fprintf(hdl,"%d\n",2);
  fprintf(hdl,"%g\n",fv->time);

  fprintf(hdl,"%d\n",fv->number_of_elements);
  fprintf(hdl,"%d\n",fv->number_of_points);

  fprintf(hdl,"%d\n",1);
  fprintf(hdl,"%d\n",1);
  fprintf(hdl,"%d\n",4);
  fprintf(hdl,"%d\n",1);

  fprintf(hdl,"%d\n",0);
  fprintf(hdl,"%d\n",0);
  fprintf(hdl,"%d\n",0);
  fprintf(hdl,"%d\n",0);
 
  for(i = 0 ; i < fv->number_of_points; i++)
    fprintf(hdl,"%f %f\n",fv->x[i],fv->y[i]);
  for(i = 0;  i < fv->number_of_elements; i++)
      fprintf(hdl,"%d %d %d\n",
                        fv->vertex[i][0],fv->vertex[i][1],fv->vertex[i][2]);
  for(i = 0;  i < fv->number_of_elements; i++)
      fprintf(hdl,"%d %d %d\n",
                fv->neighbour[i][0],fv->neighbour[i][1],fv->neighbour[i][2]);

  for(i = 0; i < fv->number_of_points; i++) {
    j = 0;
    while ((fv->vertex[j][0] != i) &&
           (fv->vertex[j][1] != i) &&
           (fv->vertex[j][2] != i)   )
      j++;
    for(k=0 ; k<4 ; k++)
      fprintf(hdl,"%f ",fv->u[j][k]);
    fprintf(hdl,"\n");
  }

  fclose(hdl);

  printf("  triangulation stored in file %s\n\n",name);
}


FVSYS *load(int const nr, char *const name, FVSYS *fv)
                                                /* load of FVSYS-struct */
{

  int fd;
  char fname[200];
  int i,j,mnp,mne;

  sprintf(fname,"FFSDATA/%s_%d.fvgr",name,nr);

  if ((fd=open(fname,O_RDONLY)) == -1) {
    perror(fname);
    return(NULL);
  }

  printf("  %s --> FVSYS-struct\n",fname);

  fv = (FVSYS *)GRAPE(FvSys,"new-instance")(name);

  read(fd, &fv->max_number_of_points, sizeof(int));
  read(fd, &fv->max_number_of_elements, sizeof(int));
  read(fd, &fv->number_of_points, sizeof(int));
  read(fd, &fv->number_of_elements,sizeof(int));

  read(fd, &NUMBER_OF_KOMPS, sizeof(int));

  fv->x = (float *)mem_alloc(sizeof(float) * fv->max_number_of_points);
  fv->y = (float *)mem_alloc(sizeof(float) * fv->max_number_of_points);

  fv->vertex = (INT3 *)
                   mem_alloc(sizeof(INT3) * fv->max_number_of_elements);
  fv->neighbour = (INT3 *)
                      mem_alloc(sizeof(INT3) * fv->max_number_of_elements);
  fv->refine = (REFINE2D *)
                      mem_alloc(sizeof(REFINE2D) * fv->max_number_of_elements);

  ASSURE(fv->x&&fv->y&&fv->vertex&&fv->neighbour&&fv->refine,
         "fv-load : can't allocate memory",return(NULL));

  fv->u = malloc_data_mem(fv->max_number_of_elements);
  fv->u_dt = malloc_data_mem(fv->max_number_of_elements);

  mnp = fv->max_number_of_points;
  mne = fv->max_number_of_elements;

  read(fd, fv->x, (mnp * sizeof(float)));
  read(fd, fv->y, (mnp * sizeof(float)));

  fv->z = NULL;

  read(fd, fv->vertex, (mne * sizeof(INT3)));
  read(fd, fv->neighbour, (mne * sizeof(INT3)));

  read(fd, fv->refine, (mne * sizeof(REFINE2D)));

  for (i=0 ; i<mne ; i++)
    read(fd, fv->u[i], (NUMBER_OF_KOMPS * sizeof(double)));

  read(fd, &fv->time, sizeof(float));
  read(fd, &fv->step, sizeof(int));
  read(fd, &fv->cfl, sizeof(float));

  close(fd);

  for (i=0 ; i<mne ; i++)
    for (j=0 ; j<NUMBER_OF_KOMPS ; j++)
    fv->u_dt[i][j] = fv->u[i][j];

  set_tr_center(fv);

  printf("   time: %f\n",fv->time);

/* to store data in ASCII-READDATA-format */
/*sprintf(fname,"%s_%d.dat",name,nr);
  write_triang(fv,fname); */

  return(fv);

}


double **malloc_data_mem (int const number_of_elements)

{

  int i;
  double **u;

  u = (double **)mem_alloc(sizeof(double *) * number_of_elements);
  ASSURE(u, "malloc_data_mem : can't allocate memory", return(NULL));
  for (i=0 ; i<number_of_elements ; i++) {
    u[i] = (double *)mem_alloc(sizeof(double) * NUMBER_OF_KOMPS);
    ASSURE(u[i], "malloc_data_mem : can't allocate memory", return(NULL));
  }

  return(u);

}


double **free_data_mem(double **u, int const number_of_elements)

{

  int i;

  for (i=0 ; i<number_of_elements ; i++) {
    mem_free(u[i],sizeof(double) * NUMBER_OF_KOMPS);
    u[i] = NULL;
  }
  mem_free(u,sizeof(double *) * number_of_elements);

  return(NULL);

}

