/**********************************************************/
/*                                                        */
/*                                                        */
/*                                                        */
/*                                                        */
/*  this demo shows how to include timedependent          */
/*  adaptive finite element in 3d                         */
/*  (the under lying data is a sequence of pre            */
/*   and post geometries)                                 */
/*                                                        */
/*                                                        */
/**********************************************************/

#include <grape.h>


static MATRIX44 perspec = {{1.0,0.0,0.0,0.0},
		           {0.0,1.0,0.0,0.0},
		           {0.0,0.0,1.0,0.0},
		           {0.0,0.0,0.0,1.0}};

FE3D *fe3d_pressure_velo()

{

  FE3D *fe;

  fe  = (FE3D *)START_METHOD(G_INSTANCE);
  ASSURE(fe,
	 "pressure-velo : can't manage classes",
	 END_METHOD(NIL));
  fe->dimension_of_value = (fe->dimension_of_value == 3)? 1 : 3;
  fe->polynomial_order = (fe->polynomial_order == 2)? 1 : 2;
  
  END_METHOD(fe);

}

main()
     
{ 

  extern FE3D *fe3d_read_ebsel_3d();
   extern INSTANCE *root_ebs_switch_send();

  TIMESCENE *tsc,*tsc2;
  SCENE *s1,*s2;
  FE3D *fe_post,*fe_pre,*fe_h;
  int start_time,frac_time,end_time;
  float main_time,fact;
  char path[50];
  char name_pre[50],name_post[50];  

static float white[3] = {1.0,1.0,1.0};

  MANAGER *mgr;
  BUTTON *butpv;
  GRAPHICDEVICE *dev;
  float zrange;

  printf("z range : ");
  scanf("%f",&zrange);
  printf("\n");
  perspec[2][2] = 1/zrange;
  printf("file format : dir/flow_pre?.dat dir/flow_post?.dat\n");
  printf("path <dir/flow_> : ");
  scanf("%s",path);
  printf("\n");
  printf("start time , end time :");
  scanf("%d %d" , &start_time, &end_time);
  printf("\n");
  printf("factor (time = step * fact) : ");
  scanf("%f" , &fact);
  printf("\n");
  
  GRAPE(Fe3d,"add-method")("read-ebsel-3d",fe3d_read_ebsel_3d);
  GRAPE(Root,"add-method")("ebs-switch-send",root_ebs_switch_send);
  
  tsc = (TIMESCENE *)GRAPE(TimeScene,"new-instance")("tsc test");
  for(frac_time = start_time;frac_time<=end_time;frac_time++) {

    main_time = fact * (frac_time - start_time);
    sprintf(name_pre,"%spre%d.dat",path,frac_time);
    sprintf(name_post,"%spost%d.dat",path,frac_time);
    printf("read %s %s\n",name_pre,name_post);

    if(frac_time == start_time) {
      printf("read %s\n",name_post);
      fe_h = fe_post = (FE3D *)GRAPE(Fe3d,"read-ebsel-3d")
	                            ("fe3d-post",name_post,1);
      ASSURE(fe_post,"can't read ebsel 3d fe",exit(1));
      tsc->dynamic = (INSTANCE *)GRAPE(TimeStep,"put")
	                              (fe_post,fe_post,main_time);
    }
    else {
      if(frac_time == end_time) {
	printf("read %s\n",name_pre);
	fe_pre = (FE3D *)GRAPE(Fe3d,"read-ebsel-3d") 
	  ("fe3d-pre",name_pre,0);
	ASSURE(fe_pre,"can't read ebsel 3d fe",exit(1));
	fe_pre->x = fe_h->x;
	fe_pre->y = fe_h->y;
	fe_pre->z = fe_h->z;
	fe_pre->vertex = fe_h->vertex;
	fe_pre->neighbour = fe_h->neighbour;
	tsc->dynamic = (INSTANCE *)GRAPE(tsc->dynamic,"put")
	  (fe_pre,fe_pre,main_time);
      }
      else {
	printf("read %s %s\n",name_pre,name_post);
	fe_pre = (FE3D *)GRAPE(Fe3d,"read-ebsel-3d") 
	  ("fe3d-pre",name_pre,0);
	fe_pre->x = fe_h->x;
	fe_pre->y = fe_h->y;
	fe_pre->z = fe_h->z;
	fe_pre->vertex = fe_h->vertex;
	fe_pre->neighbour = fe_h->neighbour;
	ASSURE(fe_pre,"can't read ebsel 3d fe",exit(1));
	fe_h = fe_post = (FE3D *)GRAPE(Fe3d,"read-ebsel-3d")
	  ("fe3d-post",name_post,1);
	ASSURE(fe_post,"can't read ebsel 3d fe",exit(1));

	printf("nop,not (pre , post) : (%d , %d) (%d , %d)\n",
	       fe_pre->number_of_points,
	       fe_post->number_of_points,
	       fe_pre->number_of_elements,
	       fe_post->number_of_elements);

	tsc->dynamic = (INSTANCE *)GRAPE(tsc->dynamic,"put")
	  (fe_pre,fe_post,main_time);
	
      }
    }
  } 

 
  dev = (GRAPHICDEVICE *)GRAPE(GraphicDevice,"get-stddev")();
  dev->transform(G_MODE_POST,G_MATRIX_PROJECT,perspec);  
/*  dev->attribute(G_MODE_SET,G_BACKGROUND_COLOR,white); */
  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();
  butpv = (BUTTON *)GRAPE(Button,"new-instance")
    ("pressure-velo",NIL,"pressure-velo");
  GRAPE(butpv,"set-pref-size")(6.,1.);
  GRAPE(mgr,"add-inter")(butpv);

  GRAPE(mgr,"handle")(tsc) ; 


}
